// board specific defines for micro monitor
#define DEFAULT_BAUD_RATE   	38400
#define MON_CPU_CLOCK       	83333333
#define MEMCLK					83333333

#define LOOPS_PER_USEC			15

// These defines allow us to keep pci code endian neutral
#define PCI_ADD8  				0
#define PCI_ADD16 				0
//#define PCI_DBG					

#define BIG_ENDIAN 		// big endian, when desired

// board specific define needed by 16c552.h
#define SIO_STEP				1	// internal 8245 UART is on 8-bit boundry

// board specific defines needed by sed135x
#define SED1356_CRT				1
#define SED_REG_BASE			0x78000000	// *RCE3
#define SED_MEM_BASE 			0x78400000
#define SED_STEP				2			// 16-bit port on 32-bit boundry

#define SED_REG16(_x_) 			*(vushort *)((ulong)SED_REG_BASE + (((ulong)_x_ * SED_STEP) ^ 2))	// Control/Status Registers, 16-bit mode
#define RD_FB16(_reg_,_val_) 	((_val_) = *((vushort *)(((ulong)SED_MEM_BASE + ((ulong)(_reg_ * SED_STEP) ^ 2)))))
#define WR_FB16(_reg_,_val_) 	(*((vushort *)(((ulong)SED_MEM_BASE + ((ulong)(_reg_ * SED_STEP)  ^ 2)))) = (_val_))
#define H2SED(_x_)				(((((ushort)_x_) & 0xff00) >> 8) | ((((ushort)_x_) & 0x00ff) << 8))
//#define H2SED(_x_)				(_x_)
//#define SED_DBG
#define SED_GET_PHYS_ADD(_reg_) (vulong)(SED_MEM_BASE + ((_reg_ * SED_STEP) ^ 2))

#define SED_DISP_MODE_CRT		1
#define SED_DISP_MODE_LCD		0

// board specific defines needed by ps2
#define PS2_BASE				0x79000000
#define PS2_STEP				4
#define PS2_REG(_x_)  			*(vuchar *)(PS2_BASE + ((_x_ * PS2_STEP) + 3))	// 8-Bit port on 32-bit boundry      
//#define PS2_DEBUG
#define PS2_TIMEOUT				100000

//-----------------------------------------------------------------------------
// MPC8245 Embedded Utilities memory Block Access Macro
// All I/O are offsets.  Note that the base address can be
// programmed to anywhere in PCI Memory Space, but we fix
// it at 0xF0000000 since there is no reason to do otherwise.
#define EUMB_BASE_ADD		0xF0000000

//-----------------------------------------------------------------------------
// MPC8245 DUART Registers are accessed as offsets into the EUMB.
//
// 16550 style DUART - use the defines in 16550.h, except for UART_DCR which 
// is setup at init time to enable the DUART.
// DUART - Channel 1
#define UART1_BASE			EUMB_BASE_ADD + 0x04500
// DUART Channel 2
#define UART2_BASE			EUMB_BASE_ADD + 0x04600

// UART_DCR controls both UARTS
#define UART_DCR			0x11	// DUART configuration register

// Bit defines for UART_DCR
#define DCR_IRQS1 			BIT3	// 0 = UART1 interrupt is passed to CPU, 1 = interrupt is routed to PCI bus.
#define DCR_IRQS2 			BIT2	// 0 = UART2 interrupt is passed to CPU, 1 = interrupt is routed to PCI bus.
#define DCR_SDM				BIT0	// 0 = Single UART with CTS/RTS handshaking, 1 = Dual UART with no handshaking

// Access macros used by cpuio.c
#define UARTA(_x_)  *(vuchar *)(UART1_BASE + _x_)
#define UARTB(_x_)  *(vuchar *)(UART2_BASE + _x_)

// CSB281 Board Control/Status Register
#define BCSR_ADD	0xff000000
#define BCSR		*(vulong *)(BCSR_ADD)
#define IDSEL1		BIT0	// 1 = enabled
#define IDSEL2		BIT1
#define IDSEL3		BIT2
#define LED0		BIT3	// 0 = enabled
#define LED1		BIT4
#define P_RST		BIT5	// 0 = reset PCI Bus
#define SMI_EN		BIT6	// 1 = PB_SW0 asserted will drive SMI to CPU
#define NMI_EN		BIT6	// 1 = PB_SW1 asserted will drive NMI to CPU
#define USER0		BIT8	// 0 = User Dip Switch 0 ON (left most switch)
#define USER1		BIT9	// 0 = User Dip Switch 1 ON
#define USER2		BIT10	// 0 = User Dip Switch 2 ON
#define USER3		BIT11	// 0 = User Dip Switch 3 ON
#define PB_SW0		BIT12	// 0 = Pushbutton Switch 0 asserted
#define PB_SW1		BIT13	// 0 = Pushbutton Switch 1 asserted

// I2C Control/Status Register
#define I2C_REG_ADD	0xff000100
#define I2C_REG		*(vulong *)(I2C_REG_ADD)
// Write control bits
#define I2C_REG_CLR_ALL 		0x00
#define I2C_REG_SET_ALL 		0xff
#define I2C_REG_CLR_SCL 		BIT0
#define I2C_REG_SET_SCL 		BIT1
#define I2C_REG_CLR_SDA			BIT2
#define I2C_REG_SET_SDA			BIT3
// Read Bit Locations
#define I2C_REG_SDA_RD			BIT0
#define I2C_REG_SCL_RD			BIT1

