/*
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:  Ed Sutter
 * email:   esutter@lucent.com
 * phone:   908-582-2351
 * 
 *
 * Modified for the CSB650 Au1100 Single Board
 * Mike Kelly - Cogent Computer Systems, Inc.
 *
 * rom_reset.s:
 */

#include "cpu.h"
#include "mips.h"
#include "au1100.h"
#include "cpuio.h"
#include "warmstart.h"

#define AU1550_MEM_ADDR		0xB4000000

#define mem_ddmode0		    0x0800
#define mem_ddaddr0	    	0x0820
#define mem_ddconfiga		0x0840
#define mem_ddconfigb		0x0848
#define mem_ddstat		    0x0850
#define mem_dderraddr		0x0858
#define mem_ddstride0		0x0860
#define mem_ddwrmd0 		0x0880
#define mem_ddprecmd  	    0x08C0
#define mem_ddautoref  	    0x08C8
#define mem_ddsref  	    0x08D0

/*
 * If ORIG_CSB655...
 * SDCS0 - 64MB MT46V16M16TG-58
 * SDCS1 - 64MB	MT46V16M16TG-58
 * SDCS2 - 64MB MT46V16M16TG-58
 * else...
 * SDCS0 - 128MB MT46V32M16TG-58
 * endif
 * NOTE: Must have Micron drive strength set to "normal" via extended
 * mode register 198MHz DDR settings
 */

#define MEM_DDMODE0_DDR		0x04276221	// DDR, 32-Bit

#ifdef ORIG_CSB655
#define MEM_DDADDR0_DDR		0xE21003F0 	// 64Mbyte at 0
#else
#define MEM_DDADDR0_DDR		0xE31003e0 	// 13 row, 10 col, 2-bank, 128Mbyte at 0
#endif

#define MEM_DDCONFIGA_DDR	0x901001f4	// Refresh Off, DCLK0 on
#define MEM_DDCONFIGB_DDR	0x00028000	// DDR Clock = 132Mhz, Full Drive
//#define MEM_DDCONFIGB_DDR	0x00008000	// DDR Clock = 132Mhz, Reduced Drive
//#define MEM_DDCONFIGB_DDR	0x00000000	// DDR Clock = 66Mhz, Reduced Drive
//#define MEM_DDCONFIGB_DDR	0x00020000	// DDR Clock = 66Mhz, Full Drive
#define MEM_DDWRMD0_DDR		0x00000063	// Write to Normal MRS
#define MEM_DDREFCFG_D_DDR	MEM_DDCONFIGA_DDR 				// refresh disable
#define MEM_DDREFCFG_E_DDR	MEM_DDCONFIGA_DDR | 0x08000000	// refresh enable

#define MEM_1MS			((396000000/1000000) * 1000)

	.bss
monStack:
	.space 4096
monStackEnd:
	.space 32

	.text
	.set noreorder

    .globl  coldstart
    .globl  warmstart
    .globl  moncomptr
    .globl  etheraddr
    .globl  ipaddr
    .globl  SPInit
    .extern start
    .extern moncom

    .text
	.global reset
	.global ipaddr
	.global etheraddr


reset:
	/* fix for big endian mode when booting from 16-bit flash */
	/* see document "Au1x00 Bus Operation".  This forces the */
	/* BE bit in stcfg0 high before any data accesses can occur. */ 
	/* this code is on litte endian mode so it works right */
	/* after reset */
	.long 0xb4003c08 # lui t0,0xb400
	.long 0x10003508 # ori t0,t0,0x1000
	.long 0x00008d09 # lw t1,0(t0)
	.long 0x02003529 # ori t1,t1,0x200
	.long 0x0000ad09 # sw t1,0(t0)
	.long 0x00000000 # nop
	.long 0x00000000 # nop
	.long 0x00000000 # nop
	.long 0x00000000 # nop
	b	reset_more
	nop
	nop

/* Provide space to allow a programmer to place an ascii
 * string in as an optional point of storage for MAC address...
 */
	.org 0x40
#include "etheraddr.S"

	.globl	alt_tfsdevtbl_base

	.balign	0x10

alt_tfsdevtbl_base:
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff
    .byte 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff

/* SR(BEV) = 1 (uncached,kseg1) exception handlers:
 * These offsets are relative to 0xbfc00000.
 * These handlers don't do much (obviously).  They simply redirect
 * the execution to reset.
 */
	.org 0x200
exc_tlbrefill:
	b	reset_more

	.org 0x280
exc_xtlbrefill:
	b	reset_more

	.org 0x300
exc_cacherr:
	b	reset_more

	.org 0x380
exc_general:
	b	reset_more

	.org 0x400
exc_interrupt:
	b	reset_more

/* End of fixed location exception entrypoints */

reset_more:
	/*
	 * Establish Status Register
	 * (set BEV, clear ERL, clear EXL, clear IE)
	 */
	li		t1, 0x00400000
	mtc0	t1, CP0_Status

	/*
	 * Establish CP0 Config0
	 * (set OD, set K0=3)
	 */
	li		t1, 0x00080003
	mtc0	t1, CP0_Config0


	/*
	 * Establish EJTAG Debug register
	 */
	mtc0	zero, CP0_Debug
	nop

	/*
	 * Establish Cause
	 * (set IV bit)
	 */
	li		t1, 0x00800000
	mtc0	t1, CP0_Cause

	/*
	 * Initialize the TLB
	 */
	li		t0, 0		 	# index value
	li		t1, 0x00000000 	# entryhi value
	li		t2, 32		   	# 32 entries
tlbloop:
	/* Probe TLB for matching EntryHi */
	mtc0	t1, CP0_EntryHi
	tlbp
	nop

	/* Examine Index[P], 1=no matching entry */
	mfc0	t3, CP0_Index
	li		t4, 0x80000000
	and		t3, t4, t3
	addiu	t1, t1, 1		# increment t1 (asid)
	beq		zero, t3, tlbloop
	nop

	/* Initialize the TLB entry */
	mtc0	t0, CP0_Index
	mtc0	zero, CP0_EntryLo0
	mtc0	zero, CP0_EntryLo1
	mtc0	zero, CP0_PageMask
	tlbwi

	/* Do it again */
	addiu	t0, t0, 1	
	bne		t0, t2, tlbloop
	nop

	/* Establish Wired (and Random) */
	mtc0	zero, CP0_Wired	// all TLB entries can be replaced
	nop

	/* Invalidate the iCache
	 */
	li		t0, (16*1024)		# iCache size;
	li		t1, 32				# line size;
	li		t2, 0x80000000		# First KSEG0 address;
	addu	t3,t0,t2			# terminate address of loop;
icloop:
	cache	0,0(t2)				# iCache indexed invalidate tag;
	addu	t2,t1				# compute next address;
	bne		t2,t3,icloop
	nop
	
	/* Invalidate the dCache
	 */
	li		t0, (16*1024)		# dCache size;
	li		t1, 32				# line size;
	li		t2, 0x80000000		# First KSEG0 address;
	addu	t3,t0,t2			# terminate address of loop;
dcloop:
	cache	1,0(t2)				# dCache indexed invalidate tag;
	addu	t2,t1				# compute next address;
	bne		t2,t3,dcloop
	nop

	/*
	 * CPU PLL Frequency = 392Mhz
	 */
	li		t0, AU1100_SYS_ADDR

	li		t1, 0x00000021
	sw		t1, sys_cpupll(t0)
	sync
	nop
	nop

	/*
	 * System Bus Divider = 3
	 */
	li		t1, 0x00000001
	sw		t1, sys_powerctrl(t0)
	sync
	/*
	 * AUX PLL Frequency = 96Mhz
	 */
	li		t1, 0x00000008
	sw		t1, sys_auxpll(t0)
	sync

	/*
	 * Start the 32kHz Oscillator
	 */
	li		t1, 0x00000100
	sw		t1, sys_cntctrl(t0)
	sync

	/*
	 * Initialize static memory controller
	 */
	li		t0, AU1100_MEM_ADDR

	/* RCE0 -
	 * E28F128 Bank 0, BE, 16-Bit, Type 3 (FLASH), 8MB @ 0x1f8000000
	 */
//	li		t1, 0x00000243  
	li		t1, 0x00046243  
	sw		t1, mem_stcfg0(t0)

	li		t1, 0x11f83fe0
	sw		t1, mem_staddr0(t0)

	/* RCE1 - Unused
	 * LCD Control CPLD, BE Type 1 (I/O), .5Mbyte @ 0x10000000
	 * Note: requires TLB setup before use
	 */
//	li		t1, 0x00000201
//	sw		t1, mem_stcfg1(t0)

//	li		t1, 0x22080b20
//	sw		t1, mem_sttime1(t0)

//	li		t1, 0x11003f00
//	sw		t1, mem_staddr1(t0)

	sync

	/* SDRAM Initialization...
	 */
/*************************************************************************/

initDRAM:

	/* wait 1mS before setup */
	li		t1, MEM_1MS
1:	add		t1, -1
	bne		t1, zero, 1b
	nop

	li		t0, AU1550_MEM_ADDR
	li		t1, MEM_DDMODE0_DDR
	sw		t1, mem_ddmode0(t0)
	
	li		t1, MEM_DDADDR0_DDR
	sw		t1, mem_ddaddr0(t0)
	sync

	li		t1, MEM_DDREFCFG_D_DDR
	sw		t1, mem_ddconfiga(t0)
	sync

	li		t1, MEM_DDCONFIGB_DDR
	sw		t1, mem_ddconfigb(t0)
	sync

	sw		zero, mem_ddprecmd(t0)
	sync

	/* Micron specific: Set normal drive strength */
	li		t1, 0x40000000
	sw		t1, mem_ddwrmd0(t0)		
	sync

	li		t1, MEM_DDWRMD0_DDR
	sw		t1, mem_ddwrmd0(t0)		
	sync

	sw		zero, mem_ddprecmd(t0)
	sync

	sw		zero, mem_ddautoref(t0)
	sync
	sw		zero, mem_ddautoref(t0)
	sync

	li		t1, MEM_DDREFCFG_E_DDR
	sw		t1, mem_ddconfiga(t0)
	sync

initDRAMdone:
	/* wait 1mS after setup */
	li		t1, MEM_1MS
1:	add		t1, -1
	bne		t1, zero, 1b
	nop

/********************************************************************/

create_stack:
	/* Create a stack using the space designated by monStackEnd, then
	 * copy the image to ram and jump into it.
	 */
	la	sp, monStackEnd
	addiu sp, -4*4

goToC:
	addiu	a0,t7,0
	la		k0, cstart
	KSEG1A(k0)
	j		k0
	nop
